<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Str;

use App\Models\PageRevision;
use App\Models\User;

class Page extends Model
{
    use HasFactory;

    protected $fillable = [
        'user_id',
        'uuid',
        'title',
        'slug',
        'content',
        'is_published',
        'main',
        'linked',
        'linkorder',
        'visibility',
    ];

    protected $casts = [
        'content' => 'array',
        'is_published' => 'boolean',
        'main' => 'boolean',
        'linked' => 'boolean',
    ];

    protected static function boot() {
        parent::boot();
        static::creating(function ($page) {
            $page->uuid = (string) Str::uuid();
        });
    }

    public function activeRevision(){
        return $this->hasOne(PageRevision::class)->where('active', true);
    }

    public function author() {
        return $this->belongsTo(User::class, 'user_id');
    }

    public function revisions() {
        return $this->hasMany(PageRevision::class);
    }
}
